;INI reading/writing wrapper functions
;Author: Shantanu Gadgil
;Enjoy!
.386
.model flat, stdcall  ;32 bit memory model
option casemap :none  ;case sensitive

include windows.inc

include kernel32.inc
include user32.inc

.const
INI_WRITEDWORD_FORMAT	db	'%lu',0
INI_WRITESDWORD_FORMAT	db	'%ld',0
INI_MAX_NUM_LENGTH		equ 16	;actually 10 digits max for a DWORD :)

.code
;Ini_GetKeyNames
;gets all the keys in the specified section
Ini_GetKeyNames	proc	pszFileName:LPCTSTR, pszSectionName:LPCTSTR, pszReturnedString:LPTSTR, nSize:SDWORD
	invoke	GetPrivateProfileSection, pszSectionName, pszReturnedString, nSize, pszFileName
	ret
Ini_GetKeyNames endp

;Ini_GetSectionNames
;gets all the section names for the specified INI file
Ini_GetSectionNames	proc	pszFileName:LPCTSTR, pszReturnedString:LPTSTR, nSize:SDWORD
	invoke	GetPrivateProfileSectionNames, pszReturnedString, nSize, pszFileName
	ret
Ini_GetSectionNames	endp

;Ini_ReadDword (obsolete)
;read a DWORD value from the INI file
;(for reading signed values (leading '+' or '-'), use Ini_ReadInt)
;returns FALSE on error
Ini_ReadDword	proc	pszFileName:LPCTSTR, pszSectionName:LPCTSTR, pszKeyName:LPCTSTR, pdwReturnValue:LPDWORD
	LOCAL	szReturnedString[INI_MAX_NUM_LENGTH]:BYTE
	
	push	ebx
	
	invoke	GetPrivateProfileString, pszSectionName, pszKeyName, NULL, addr szReturnedString,
		INI_MAX_NUM_LENGTH, pszFileName
	
	lea	eax,szReturnedString
	.if	byte ptr [eax] == 0
		mov	eax,FALSE
		ret
	.endif

	;returns the dword in eax
	invoke	SM_DecimalStringToDword, addr szReturnedString
	
	mov	ebx, pdwReturnValue
	mov	[ebx],eax
	
	pop	ebx
	
	mov	eax,TRUE 
	ret
Ini_ReadDword endp

;Ini_ReadInt
;like Ini_ReadDword, except that it supports signed numbers
;returns FALSE on error
Ini_ReadInt	proc	pszFileName:LPCTSTR, pszSectionName:LPCTSTR, pszKeyName:LPCTSTR, pdwReturnValue:LPDWORD, bSigned:BOOL
	LOCAL	szReturnedString[INI_MAX_NUM_LENGTH]:BYTE
	
	push	ebx
	
	invoke	GetPrivateProfileString, pszSectionName, pszKeyName, NULL, addr szReturnedString,
		INI_MAX_NUM_LENGTH, pszFileName
	
	lea	eax,szReturnedString
	.if	byte ptr [eax] == 0
		mov	eax,FALSE
		ret
	.endif

	;returns the number in location at 'pdwReturnValue' :) :)
	invoke	SM_DecimalStringToInt, addr szReturnedString, pdwReturnValue, bSigned

	pop	ebx
	
	mov	eax,TRUE 
	ret
Ini_ReadInt endp

;Ini_ReadString
;reads a string from the INI.
;returns FALSE if read failed
;TRUE otherwise
Ini_ReadString	proc	pszFileName:LPCTSTR, pszSectionName:LPCTSTR, pszKeyName:LPCTSTR, pszReturnedString:LPTSTR, nSize:SDWORD
	invoke	GetPrivateProfileString, pszSectionName, pszKeyName, NULL, pszReturnedString, nSize, pszFileName
	
	mov	eax,pszReturnedString
	.if	byte ptr [eax] == 0	;NULL string read
		mov	eax, FALSE
	.else
		mov	eax, TRUE
	.endif
	
	ret
Ini_ReadString endp

;Ini_WriteString
;writes a string for the specified KEY in the specified SECTION
Ini_WriteString	proc	pszFileName:LPCTSTR, pszSectionName:LPCTSTR, pszKeyName:LPCTSTR, pszValueString:LPCTSTR
	invoke	WritePrivateProfileString, pszSectionName, pszKeyName, pszValueString, pszFileName
	ret
Ini_WriteString endp

;Ini_WriteInt
;writes an integer value in the INI. supports signed numbers
Ini_WriteInt	proc	pszFilename:LPCTSTR, pszSectionName:LPCTSTR, pszKeyName:LPCTSTR, iValue:SDWORD, bSigned:BOOL
	LOCAL	szTemp[16]:BYTE
	
	.if	bSigned == TRUE
		mov	eax, offset INI_WRITESDWORD_FORMAT
	.else
		mov	eax, offset INI_WRITEDWORD_FORMAT
	.endif
	invoke	wsprintf, addr szTemp, eax, iValue
	invoke	WritePrivateProfileString, pszSectionName, pszKeyName, addr szTemp, pszFilename
	ret
Ini_WriteInt	endp

end
